/*
 *    GS3MGRetrieve.java
 *    Copyright (C) 2002 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.service;


// Greenstone classes
import org.greenstone.mg.*;
import org.greenstone.gsdl3.util.*;

// XML classes
import org.w3c.dom.Element;

// General Java classes
import java.io.File;


/** The content retrieval class for GS3 building
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @version $Revision: 8689 $
 */

public class GS3MGRetrieve
    extends ServiceRack {
    // the services on offer
    protected static final String DOCUMENT_CONTENT_RETRIEVE_SERVICE = "DocumentContentRetrieve";
    
    // Parameters used
    private static final String INDEX_PARAM = "index";

    // Elements used in the config file that are specific to this class
    private static final String DEFAULT_INDEX_ELEM = "defaultIndex";

    private MGWrapper mg_src = null;
    protected SQLQuery database = null;
    
    protected Element config_info = null; // the xml from the config file

    private String default_index = null;

    /** constructor */
    public GS3MGRetrieve()
    {
	this.database = new SQLQuery();
	this.mg_src = new MGWrapper();
    }


    /** configure this service */
    public boolean configure(Element info, Element extra_info)
    {
	this.config_info = info;
	// Do specific configuration
	System.out.println("Configuring GS3MGRetrieve...");
	// System.out.println("info:\n" + converter_.getString(info));
	// System.out.println("extra_info:\n" + converter_.getString(extra_info));

	// Get the default index out of <defaultIndex> (buildConfig.xml)
	Element def = (Element) GSXML.getChildByTagName(info, DEFAULT_INDEX_ELEM);
	if (def != null) {
	    this.default_index = def.getAttribute(GSXML.NAME_ATT);
	}
	if (this.default_index == null || this.default_index.equals("")) {
	    System.err.println("Error: default index not specified!");
	    return false;
	}

	// System.out.println("Default index: " + this.default_index);
	Element dcr_service = this.doc.createElement(GSXML.SERVICE_ELEM);
	dcr_service.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_RETRIEVE);
	dcr_service.setAttribute(GSXML.NAME_ATT, DOCUMENT_CONTENT_RETRIEVE_SERVICE);
	this.short_service_info.appendChild(dcr_service);

	// check that site_home is set
	if (this.site_home == null || this.site_home.equals("")) {
	    System.err.println("GS3Retrieve Error: site_home is not set, so cannot work out the site name and cannot determine the database name");
	    return false;
	}
	String site_name = this.site_home.substring(this.site_home.lastIndexOf(File.separator)+1);
	if (site_name.equals("")) {
	    System.err.println("GS3Retrieve Error: Cannot extract the site name from site home: "+this.site_home);
	    return false;
	}
	if (!database.setDatabase(site_name+"_"+this.cluster_name)) {
	    System.err.println("GS3Retrieve Error: Could not open SQL database!");
	    return false;
	}

	// look for document display format
	String path = GSPath.appendLink(GSXML.DISPLAY_ELEM, GSXML.FORMAT_ELEM);
	Element display_format = (Element)GSXML.getNodeByPath(extra_info, path);
	if (display_format != null) {
	    this.format_info_map.put(DOCUMENT_CONTENT_RETRIEVE_SERVICE, this.doc.importNode(display_format, true));
	    // should we make a copy?
	}

	return true;
    }

    protected Element getServiceDescription(String service_id, String lang, String subset) {
	if (service_id.equals(DOCUMENT_CONTENT_RETRIEVE_SERVICE)) {
	    Element dcr_service = this.doc.createElement(GSXML.SERVICE_ELEM);
	    dcr_service.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_RETRIEVE);
	    dcr_service.setAttribute(GSXML.NAME_ATT, DOCUMENT_CONTENT_RETRIEVE_SERVICE);
	    return dcr_service;
	}

	return null;
    }
    /** Retrieve the content of a document */
    protected Element processDocumentContentRetrieve(Element request)
    {
	// Create a new (empty) result message
	Element result = this.doc.createElement(GSXML.RESPONSE_ELEM);
	result.setAttribute(GSXML.FROM_ATT, DOCUMENT_CONTENT_RETRIEVE_SERVICE);
	result.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_PROCESS);

	// Get the parameters of the request - no parameters at this stage
  	//Element param_list = (Element) GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);

	// Get the request content 
	
	Element query_doc_list = (Element) GSXML.getChildByTagName(request, GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	if (query_doc_list == null) {
	    System.err.println("Error: DocumentContentRetrieve request specified no doc nodes.\n");
	    return result;
	}
	
	Element doc_list = this.doc.createElement(GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	result.appendChild(doc_list);
	
	// The location of the MG index and text files
	String basedir = GSFile.collectionBaseDir(this.site_home, this.cluster_name) + File.separatorChar;  // Needed by MG
	String textdir = GSFile.collectionTextPath("index");//this.cluster_name);
	// index is only needed to start up MG, not used so just use the default index
	String indexpath = GSFile.collectionIndexPath("index"/*this.cluster_name*/, this.default_index);
	this.mg_src.setIndex(indexpath);
	
	// Get the documents
	String[] doc_ids = GSXML.getAttributeValuesFromList(query_doc_list, 
							     GSXML.NODE_ID_ATT);
	for (int i = 0; i < doc_ids.length; i++) {
	    //ystem.out.println("getting doc id "+doc_ids[i]);
	    String doc_id = doc_ids[i];
	    if (GS3OID.needsTranslating(doc_id)) {
	    //doc_id = this.gdbm_src.translateOID(doc_id);
		doc_id = GS3OID.translateOID(doc_id);
		//ystem.out.println("translated id = "+doc_id);
	    }
	    if (GS3OID.isDocTop(doc_id) && database.isHierarchicalDocument(doc_id)) {
		// if we have a whole doc id, and the document is hierarchical,
		// we want to change the id to be the top id of the section 
		// hierarchy
	    	doc_id = GS3OID.createOID(doc_id, "1");
	    }
	    String doc_num = this.database.OID2MGNum(doc_id);	    
	    // doc nums have the index prefixed 
	    doc_num = doc_num.substring(doc_num.indexOf(".")+1);
	    int doc_int = Integer.parseInt(doc_num);
	    
	    String doc_content = "";
	    try {
		doc_content = this.mg_src.getDocument(basedir, textdir, doc_int);
	    } catch (Exception e) {
		System.out.println("exception happended with mg_src.getDocument()");
		doc_content = "this is the content for section hash id "+ doc_id+", mg doc num "+doc_int+"\n";
	    }
	    
	    //ystem.out.println("Doc content: " + doc_content + "|");
	    //ystem.out.println("Doc ID: " + doc_id);
	    // remove any ctrl-c or ctrl-b (I hope these are the right codes)
	    doc_content = doc_content.replaceAll("\u0002|\u0003", "");
	    // replace _httpimg_ with the correct address
	    //doc_content = resolveImages(doc_content, doc_id);
	    // Stick it in a text node
	    Element doc = this.doc.createElement(GSXML.DOC_NODE_ELEM);
	    doc.setAttribute(GSXML.NODE_ID_ATT, doc_id);
	    GSXML.addDocText(this.doc, doc, doc_content);
	    Element nodeContent = (Element)GSXML.getChildByTagName(doc, "nodeContent");
	    doc_list.appendChild(doc);
	}

	return result;
    }

}
